@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/copy.c,v 1.17 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/copy.c,v 1.19 2003/08/31 11:11:00 werner Exp $
  *
  * Kopieren von Dateien / Verzeichnissen
  *
@@ -37,6 +37,8 @@ int CopyFile(Statistic *statistic_ptr,
   int         term;
   int         result;
   DIR         *tmpdir = NULL;
+  int	      refresh_dirwindow = FALSE;
+
 
   result = -1;
 
@@ -103,6 +105,10 @@ int CopyFile(Statistic *statistic_ptr,
                 MESSAGE( message );
                 return( result );
         }
+	else
+	{
+		refresh_dirwindow = TRUE;
+	}
      }
      else
      {
@@ -237,9 +243,16 @@ int CopyFile(Statistic *statistic_ptr,
     result = 0;
   }
 
+  if( refresh_dirwindow) 
+  {
+  	RefreshDirWindow();
+  }
+
 FNC_XIT:
  
+  move( LINES - 3, 1 ); clrtoeol();
   move( LINES - 2, 1 ); clrtoeol();
+  move( LINES - 1, 1 ); clrtoeol();
 
   return( result );
 }
@@ -265,28 +278,25 @@ int GetCopyParameter(char *from_file, BOOL path_copy, char *to_file, char *to_di
 
   if( path_copy )
   {
-    (void) sprintf( buffer, "PATHCOPY %s AS", from_file );
+    (void) sprintf( buffer, "PATHCOPY %s", from_file );
   }
   else
   {
-    (void) sprintf( buffer, "COPY %s AS", from_file );
+    (void) sprintf( buffer, "COPY %s", from_file );
   }
 
   ClearHelp();
 
-  MvAddStr( LINES - 2, 1, buffer );
+  MvAddStr( LINES - 3, 1, buffer );
+  MvAddStr( LINES - 2, 1, "AS   ");
 
-  if( InputString(to_file, LINES - 2, strlen(buffer) + 2, COLS - strlen(buffer) - 3, "\r\033" ) != CR)
-      return( -1 );
-
-  x = 1 + strlen( buffer ) + 1 + strlen( to_file ) + 1;
-
-  MvAddStr( LINES - 2, x, "TO " ); 
-  x += 3;
-  if( InputString( to_dir, LINES - 2, x, COLS - x - 1, "\r\033" ) != CR ) 
-    return( -1 );
-
-  return( 0 );
+  if( InputString(to_file, LINES - 2, 6, COLS - 6, "\r\033" ) == CR){
+    MvAddStr( LINES - 1, 1, "TO   " ); 
+    if( InputString( to_dir, LINES - 1, 6, COLS - 6, "\r\033" ) == CR ) 
+    return( 0 );
+  }
+  ClearHelp();
+  return( -1 );
 }
 
 
