@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/dirwin.c,v 1.20 2001/04/16 13:34:26 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/dirwin.c,v 1.23 2003/08/31 11:11:00 werner Exp $
  *
  * Funktionen zur Handhabung des DIR-Windows
  *
@@ -139,28 +139,28 @@ static void PrintDirEntry(WINDOW *win,
   for(j=0; j < dir_entry_list[entry_no].level; j++) 
   {
     if( dir_entry_list[entry_no].indent & ( 1L << j ) )
-      (void) strcat( buffer, "|  " );
+      (void) strcat( buffer, "| " );
     else
-      (void) strcat( buffer, "   " );
+      (void) strcat( buffer, "  " );
   }
   de_ptr = dir_entry_list[entry_no].dir_entry;
   if( de_ptr->next )
-    (void) strcat( buffer, "6--" );
+    (void) strcat( buffer, "6-" );
   else
-    (void) strcat( buffer, "3--" );
+    (void) strcat( buffer, "3-" );
 
   switch( dir_mode )
   {
     case MODE_1 : 
                  (void)GetAttributes(de_ptr->stat_struct.st_mode, attributes);
                  (void)CTime( de_ptr->stat_struct.st_mtime, modify_time );
-                 if ((line_buffer = (char *) malloc(36)) == NULL)
+                 if ((line_buffer = (char *) malloc(38)) == NULL)
                  {
                     ERROR_MSG("malloc() Failed*Abort");
                     exit(1);
                  }
 #ifdef HAS_LONGLONG
-                 (void) strcpy( format, "%10s %3d %7lld %12s");
+                 (void) strcpy( format, "%10s %3d %8lld %12s");
                  
 		 (void) sprintf( line_buffer, format, attributes,
                                  de_ptr->stat_struct.st_nlink,
@@ -168,7 +168,7 @@ static void PrintDirEntry(WINDOW *win,
                                  modify_time
                                  );
 #else
-                 (void) strcpy( format, "%10s %3d %7d %12s");
+                 (void) strcpy( format, "%10s %3d %8d %12s");
                  (void) sprintf( line_buffer, format, attributes,
                                  de_ptr->stat_struct.st_nlink,
                                  de_ptr->stat_struct.st_size,
@@ -190,12 +190,12 @@ static void PrintDirEntry(WINDOW *win,
                      (void) sprintf( group, "%d", (int) de_ptr->stat_struct.st_gid );
                      group_name_ptr = group;
                  }
-                 if ((line_buffer = (char *) malloc(37)) == NULL)
+                 if ((line_buffer = (char *) malloc(39)) == NULL)
                  {
                     ERROR_MSG("malloc() Failed*Abort");
                     exit(1);
                  }
-                 (void) strcpy( format, "%10s %6d  %-8s %-8s");
+                 (void) strcpy( format, "%10s %8d  %-8s %-8s");
                  (void) sprintf( line_buffer, format, 
                                  attributes, 
                                  de_ptr->stat_struct.st_ino, 
@@ -207,7 +207,7 @@ static void PrintDirEntry(WINDOW *win,
                  (void) CTime( de_ptr->stat_struct.st_ctime, change_time );
                  (void) CTime( de_ptr->stat_struct.st_atime, access_time );
                  (void) strcpy( format, "Chg.: %12s  Acc.: %12s");
-                 if ((line_buffer = (char *) malloc(38+1)) == NULL)
+                 if ((line_buffer = (char *) malloc(40)) == NULL)
                  {
                     ERROR_MSG("malloc() Failed*Abort");
                     exit(1);
@@ -225,8 +225,8 @@ static void PrintDirEntry(WINDOW *win,
      WbkgdSet(win, COLOR_PAIR(color)| A_BOLD );
      if(line_buffer) {
        aux = strlen(line_buffer);
-       if(window_width < aux) {
-         line_buffer[window_width] = '\0';
+       if(window_width <= aux) {
+         line_buffer[window_width - 1] = '\0';
          suppress_output = TRUE;
        }
        mvwaddstr(win, y, 0, line_buffer );
@@ -237,8 +237,8 @@ static void PrintDirEntry(WINDOW *win,
     /* Output Graph */
     l1 = strlen(buffer);
     n = window_width - aux;
-    if(l1 > n) {
-       buffer[MAXIMUM(n, 0)] = '\0';
+    if((int)l1 > n) {
+       buffer[MAXIMUM(n-1, 0)] = '\0';
        suppress_output = TRUE;
     }
     PrintSpecialString( win, y, aux, buffer, color );
@@ -265,15 +265,18 @@ static void PrintDirEntry(WINDOW *win,
 
     n = window_width - aux - l1;
     l2 = strlen(buffer);
-    if(l2 > n)
-      buffer[MAXIMUM(n, 0)] = '\0';
+    if((int)l2 > n)
+      buffer[MAXIMUM(n-1, 0)] = '\0';
 
-    waddstr( win, buffer );
+/*    waddstr( win, buffer );*/
+    mvwaddstr( win, y, aux + l1, buffer);
     WbkgdSet(win, COLOR_PAIR(color)|A_BOLD);
 
 #else
     if( hilight ) wattrset( win, A_REVERSE );
-    waddstr( win, buffer );
+/*    waddstr( win, buffer );*/
+    mvwaddstr( win, y, aux + l1, buffer);
+
     if( hilight ) wattrset( win, 0 );
 #endif /* COLOR_SUPPORT */
 #endif /* NO_HIGHLIGHT */
@@ -305,7 +308,7 @@ void DisplayTree(WINDOW *win, int start_entry_no, int hilight_no)
 
   for(i=0; i < height; i++)
   {
-    if( start_entry_no + i >= statistic.disk_total_directories ) break;
+    if( start_entry_no + i >= (int)statistic.disk_total_directories ) break;
  
     if( start_entry_no + i != hilight_no )
       PrintDirEntry( win, start_entry_no + i, i, FALSE );
@@ -541,6 +544,274 @@ static void Moveppage(int *disp_begin_pos, int *cursor_pos, DirEntry **dir_entry
    return;
 }
 
+void MoveEnd(DirEntry **dir_entry)
+{
+    statistic.disp_begin_pos = MAXIMUM(0, total_dirs - window_height);
+    statistic.cursor_pos     = total_dirs - statistic.disp_begin_pos - 1;
+    *dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+    (*dir_entry)->start_file = 0;
+    (*dir_entry)->cursor_pos = -1;
+    DisplayFileWindow( *dir_entry );
+    RefreshWindow( file_window );
+    DisplayTree( dir_window, statistic.disp_begin_pos, 
+		statistic.disp_begin_pos + statistic.cursor_pos);
+    return;
+}
+
+void MoveHome(DirEntry **dir_entry)
+{
+    if( statistic.disp_begin_pos == 0 && statistic.cursor_pos == 0 )
+    {  /* Position 1 bereits errecht */
+       /*----------------------------*/
+       beep();
+    }
+    else
+    {
+       statistic.disp_begin_pos = 0;
+       statistic.cursor_pos     = 0;
+       *dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+       (*dir_entry)->start_file = 0;
+       (*dir_entry)->cursor_pos = -1;
+       DisplayFileWindow( *dir_entry );
+       RefreshWindow( file_window );
+       DisplayTree( dir_window, statistic.disp_begin_pos, 
+		statistic.disp_begin_pos + statistic.cursor_pos);
+    }
+    return;
+}
+
+void HandlePlus(DirEntry *dir_entry, DirEntry *de_ptr, char *new_login_path,
+		DirEntry *start_dir_entry, BOOL *need_dsp_help)
+{
+    if( !dir_entry->not_scanned ) {
+        beep();
+    } else {
+	for( de_ptr=dir_entry->sub_tree; de_ptr; de_ptr=de_ptr->next) {
+	    GetPath( de_ptr, new_login_path );
+	    ReadTree( de_ptr, new_login_path, 0 );
+	    SetMatchingParam( de_ptr );
+	}
+	dir_entry->not_scanned = FALSE;
+	BuildDirEntryList( start_dir_entry );
+	DisplayTree( dir_window, statistic.disp_begin_pos, 
+                 statistic.disp_begin_pos + statistic.cursor_pos );
+	DisplayDiskStatistic();
+	DisplayAvailBytes();
+	*need_dsp_help = TRUE;
+    }
+}
+
+void HandleReadSubTree(DirEntry *dir_entry, DirEntry *start_dir_entry,
+		       BOOL *need_dsp_help)
+{
+    ScanSubTree( dir_entry );
+    BuildDirEntryList( start_dir_entry );
+    DisplayTree( dir_window, statistic.disp_begin_pos, 
+		 statistic.disp_begin_pos + statistic.cursor_pos );
+    DisplayDiskStatistic();
+    DisplayAvailBytes();
+    *need_dsp_help = TRUE;
+}
+
+void HandleUnreadSubTree(DirEntry *dir_entry, DirEntry *de_ptr,
+			 DirEntry *start_dir_entry, BOOL *need_dsp_help)
+{
+    if( dir_entry->not_scanned || (dir_entry->sub_tree == NULL) ) {
+	beep();
+    } else {
+	for( de_ptr=dir_entry->sub_tree; de_ptr; de_ptr=de_ptr->next) {
+	    UnReadTree( de_ptr );
+	}
+	dir_entry->not_scanned = TRUE;
+	BuildDirEntryList( start_dir_entry );
+        DisplayTree( dir_window, statistic.disp_begin_pos, 
+		    statistic.disp_begin_pos + statistic.cursor_pos );
+        DisplayAvailBytes();
+        *need_dsp_help = TRUE;
+    }
+    return;
+}
+
+void HandleTagDir(DirEntry *dir_entry, BOOL value)
+{
+    FileEntry *fe_ptr;
+    for(fe_ptr=dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
+    {
+	if( (fe_ptr->matching) && (fe_ptr->tagged != value ))
+	{
+	    fe_ptr->tagged = value;
+	    if (value)
+	    {
+		dir_entry->tagged_files++;
+		dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
+	       	statistic.disk_tagged_files++;
+		statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
+	    }else{
+		dir_entry->tagged_files--;
+		dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
+	       	statistic.disk_tagged_files--;
+		statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
+	    }
+	}
+    }
+    dir_entry->start_file = 0;
+    dir_entry->cursor_pos = -1;
+    DisplayFileWindow( dir_entry );
+    RefreshWindow( file_window );
+    DisplayDiskStatistic();
+    return;
+}
+
+void HandleTagAllDirs(DirEntry *dir_entry, BOOL value )
+{
+    FileEntry *fe_ptr;
+    long i;
+    for(i=0; i < total_dirs; i++)
+    {
+	for(fe_ptr=dir_entry_list[i].dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
+	{
+	    if( (fe_ptr->matching) && (fe_ptr->tagged != value) )
+	    if (value)
+	    {
+		fe_ptr->tagged = value;
+	       	dir_entry->tagged_files++;
+		dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
+	       	statistic.disk_tagged_files++;
+		statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
+	    }else{
+		fe_ptr->tagged = value;
+	       	dir_entry->tagged_files--;
+		dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
+	       	statistic.disk_tagged_files--;
+		statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
+	    }
+	}
+    }
+    dir_entry->start_file = 0;
+    dir_entry->cursor_pos = -1;
+    DisplayFileWindow( dir_entry );
+    RefreshWindow( file_window );
+    DisplayDiskStatistic();
+    return;
+}
+
+void HandleShowAllTagged(DirEntry *dir_entry,DirEntry *start_dir_entry, BOOL *need_dsp_help, int *ch)
+{
+    if( statistic.disk_tagged_files )
+    {
+	if(dir_entry->login_flag) 
+	{
+	    dir_entry->login_flag = FALSE;
+	} else {
+	    dir_entry->big_window  = TRUE;
+            dir_entry->global_flag = TRUE;
+	    dir_entry->tagged_flag = TRUE;
+	    dir_entry->start_file  = 0;
+	    dir_entry->cursor_pos  = 0;
+	}
+	if( HandleFileWindow(dir_entry) != LOGIN_ESC )
+	{
+	    DisplayDiskStatistic();
+	    dir_entry->start_file = 0;
+	    dir_entry->cursor_pos = -1;
+            DisplayFileWindow( dir_entry );
+            RefreshWindow( small_file_window );
+            RefreshWindow( big_file_window );
+	    BuildDirEntryList( start_dir_entry );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos);
+	}else{
+	    BuildDirEntryList( statistic.tree );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos);
+	    *ch = 'L';
+	}
+    }else{
+	dir_entry->login_flag = FALSE;
+	beep();
+    }
+    *need_dsp_help = TRUE;
+    return;
+}
+
+void HandleShowAll(DirEntry *dir_entry, DirEntry *start_dir_entry, BOOL *need_dsp_help, int *ch)
+{
+    if( statistic.disk_matching_files )
+    {
+	if(dir_entry->login_flag) 
+	{
+	    dir_entry->login_flag = FALSE;
+	} else {
+	    dir_entry->big_window  = TRUE;
+	    dir_entry->global_flag = TRUE;
+	    dir_entry->start_file  = 0;
+	    dir_entry->cursor_pos  = 0;
+	}
+	if( HandleFileWindow(dir_entry) != LOGIN_ESC )
+	{
+	    DisplayDiskStatistic();
+	    dir_entry->start_file = 0;
+	    dir_entry->cursor_pos = -1;
+            DisplayFileWindow( dir_entry );
+            RefreshWindow( small_file_window );
+            RefreshWindow( big_file_window );
+	    BuildDirEntryList( start_dir_entry );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos);
+	} else {
+	    BuildDirEntryList( statistic.tree );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos );
+	    *ch = 'L';
+	}
+    } else {
+	dir_entry->login_flag = FALSE;
+	beep();
+    }
+    *need_dsp_help = TRUE;
+    return;
+}
+
+void HandleSwitchWindow(DirEntry *dir_entry, DirEntry *start_dir_entry, BOOL *need_dsp_help, int *ch)
+{
+    if( dir_entry->matching_files )
+    {
+	if(dir_entry->login_flag) 
+	{
+	    dir_entry->login_flag = FALSE;
+	} else {
+	    dir_entry->global_flag = FALSE;
+            dir_entry->tagged_flag = FALSE;
+	    dir_entry->big_window  = bypass_small_window;
+	    dir_entry->start_file  = 0;
+	    dir_entry->cursor_pos  = 0;
+	}
+	if( HandleFileWindow( dir_entry ) != LOGIN_ESC )
+        {
+	    dir_entry->start_file = 0;
+	    dir_entry->cursor_pos = -1;
+	    DisplayFileWindow( dir_entry );
+            RefreshWindow( small_file_window );
+            RefreshWindow( big_file_window );
+	    BuildDirEntryList( start_dir_entry );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos);
+	    DisplayDiskStatistic();
+	} else {
+	    BuildDirEntryList( statistic.tree );
+            DisplayTree( dir_window, statistic.disp_begin_pos, 
+			statistic.disp_begin_pos + statistic.cursor_pos);
+	    *ch = 'L';
+	}
+	DisplayAvailBytes();
+	*need_dsp_help = TRUE;
+    } else {
+	dir_entry->login_flag = FALSE;
+	beep();
+    }
+    return;
+}
+
 
 int HandleDirWindow(DirEntry *start_dir_entry)
 {
@@ -586,7 +857,7 @@ int HandleDirWindow(DirEntry *start_dir_entry)
       else {            /* Entry of form "beta" or "/full/path/alpha/beta" */
         strcpy(new_login_path, initial_directory);
       }
-      for ( i = 0; i < statistic.disk_total_directories; i++ )
+      for ( i = 0; i < (int)statistic.disk_total_directories; i++ )
       {
         if ( *new_login_path == FILE_SEPARATOR_CHAR )
           GetPath( dir_entry_list[i].dir_entry, new_name );
@@ -696,84 +967,23 @@ int HandleDirWindow(DirEntry *start_dir_entry)
                      break;
       case KEY_PPAGE:Moveppage(&statistic.disp_begin_pos, &statistic.cursor_pos, &dir_entry);
                      break;
-      case KEY_HOME: if( statistic.disp_begin_pos == 0 && statistic.cursor_pos == 0 )
-		     { 
-		       /* Position 1 bereits errecht */
-		       /*----------------------------*/
-		       beep();
-		     }
-		     else
-		     {
-		       statistic.disp_begin_pos = 0;
-		       statistic.cursor_pos     = 0;
-		       dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
-		       dir_entry->start_file = 0;
-		       dir_entry->cursor_pos = -1;
-                       DisplayFileWindow( dir_entry );
-                       RefreshWindow( file_window );
-                       DisplayTree( dir_window, statistic.disp_begin_pos, 
-				    statistic.disp_begin_pos + statistic.cursor_pos 
-				  );
-		     }
-                     break;
-      case KEY_END : statistic.disp_begin_pos = MAXIMUM(0, total_dirs - window_height);
-		     statistic.cursor_pos     = total_dirs - statistic.disp_begin_pos - 1;
-		     dir_entry = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
-		     dir_entry->start_file = 0;
-		     dir_entry->cursor_pos = -1;
-                     DisplayFileWindow( dir_entry );
-                     RefreshWindow( file_window );
-                     DisplayTree( dir_window, statistic.disp_begin_pos, 
-				  statistic.disp_begin_pos + statistic.cursor_pos 
-				);
+      case KEY_HOME: MoveHome(&dir_entry);
                      break;
+      case KEY_END : MoveEnd(&dir_entry);
+    		     break;
       case KEY_RIGHT:
-      case '+':       if( !dir_entry->not_scanned ) {
-                        beep();
-		      } else {
-		        for( de_ptr=dir_entry->sub_tree; de_ptr; de_ptr=de_ptr->next) {
-		          GetPath( de_ptr, new_login_path );
-		          ReadTree( de_ptr, new_login_path, 0 );
-			  SetMatchingParam( de_ptr );
-		        }
-		        dir_entry->not_scanned = FALSE;
-	                BuildDirEntryList( start_dir_entry );
-                        DisplayTree( dir_window, statistic.disp_begin_pos, 
-		                     statistic.disp_begin_pos + statistic.cursor_pos 
-		                   );
-			DisplayDiskStatistic();
-	                DisplayAvailBytes();
-	                need_dsp_help = TRUE;
-		      }
-	              break;
+      case '+':      HandlePlus(dir_entry, de_ptr, new_login_path,
+    				start_dir_entry, &need_dsp_help);
+	             break;
       case '\t':
-      case '*':       ScanSubTree( dir_entry );
-	              BuildDirEntryList( start_dir_entry );
-                      DisplayTree( dir_window, statistic.disp_begin_pos, 
-		                   statistic.disp_begin_pos + statistic.cursor_pos 
-		                 );
-		      DisplayDiskStatistic();
-	              DisplayAvailBytes();
-	              need_dsp_help = TRUE;
-	              break;
+      case '*':      HandleReadSubTree(dir_entry, start_dir_entry,
+    					&need_dsp_help);
+    		     break;
       case KEY_LEFT:
       case '-':
-      case KEY_BTAB:  if( dir_entry->not_scanned ||
-                         (dir_entry->sub_tree == NULL) ) {
-                        beep();
-		      } else {
-		        for( de_ptr=dir_entry->sub_tree; de_ptr; de_ptr=de_ptr->next) {
-		          UnReadTree( de_ptr );
-			}
-		        dir_entry->not_scanned = TRUE;
-	                BuildDirEntryList( start_dir_entry );
-                        DisplayTree( dir_window, statistic.disp_begin_pos, 
-		                     statistic.disp_begin_pos + statistic.cursor_pos 
-		                   );
-	                DisplayAvailBytes();
-	                need_dsp_help = TRUE;
-		      }
-	              break;
+      case KEY_BTAB: HandleUnreadSubTree(dir_entry, de_ptr, start_dir_entry,
+    					 &need_dsp_help);
+	             break;
       case 'F':
       case 'f':      if(ReadFileSpec() == 0) {
 		       dir_entry->start_file = 0;
@@ -785,101 +995,17 @@ int HandleDirWindow(DirEntry *start_dir_entry)
 		     need_dsp_help = TRUE;
                      break;
       case 'T' :
-      case 't' :     for(fe_ptr=dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
-		     {
-		       if( fe_ptr->matching ) 
-		       {
-			 if( !fe_ptr->tagged )
-			 {
-			   fe_ptr->tagged = TRUE;
-	       	           dir_entry->tagged_files++;
-		           dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
-	       	           statistic.disk_tagged_files++;
-		           statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
-			 }
-		       }
-		     }
-		     dir_entry->start_file = 0;
-		     dir_entry->cursor_pos = -1;
-                     DisplayFileWindow( dir_entry );
-                     RefreshWindow( file_window );
-		     DisplayDiskStatistic();
-		     break;
+      case 't' :     HandleTagDir(dir_entry, TRUE);
+    		     break;
 
       case 'U' :
-      case 'u' :     for(fe_ptr=dir_entry->file; fe_ptr; fe_ptr=fe_ptr->next)
-		     {
-		       if( fe_ptr->matching ) 
-		       {
-			 if( fe_ptr->tagged )
-			 {
-			   fe_ptr->tagged = FALSE;
-	       	           dir_entry->tagged_files--;
-		           dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
-	       	           statistic.disk_tagged_files--;
-		           statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
-			 }
-		       }
-		     }
-		     dir_entry->start_file = 0;
-		     dir_entry->cursor_pos = -1;
-                     DisplayFileWindow( dir_entry );
-                     RefreshWindow( file_window );
-		     DisplayDiskStatistic();
-		     break;
+      case 'u' :     HandleTagDir(dir_entry, FALSE);
+    		     break;
       case 'T' & 0x1F :
-		     for(i=0; i < total_dirs; i++)
-		     {
-                       for(fe_ptr=dir_entry_list[i].dir_entry->file; 
-			   fe_ptr; 
-			   fe_ptr=fe_ptr->next
-			  )
-		       {
-		         if( fe_ptr->matching ) 
-		         {
-			   if( !fe_ptr->tagged )
-			   {
-			     fe_ptr->tagged = TRUE;
-	       	             dir_entry->tagged_files++;
-		             dir_entry->tagged_bytes += fe_ptr->stat_struct.st_size;
-	       	             statistic.disk_tagged_files++;
-		             statistic.disk_tagged_bytes += fe_ptr->stat_struct.st_size;
-			   }
-			 }
-		       }
-		     }
-		     dir_entry->start_file = 0;
-		     dir_entry->cursor_pos = -1;
-                     DisplayFileWindow( dir_entry );
-                     RefreshWindow( file_window );
-		     DisplayDiskStatistic();
+                     HandleTagAllDirs(dir_entry,TRUE);
 		     break;
       case 'U' & 0x1F :
-		     for(i=0; i < total_dirs; i++)
-		     {
-                       for(fe_ptr=dir_entry_list[i].dir_entry->file; 
-			   fe_ptr; 
-			   fe_ptr=fe_ptr->next
-			  )
-		       {
-		         if( fe_ptr->matching ) 
-		         {
-			   if( fe_ptr->tagged )
-			   {
-			     fe_ptr->tagged = FALSE;
-	       	             dir_entry->tagged_files--;
-		             dir_entry->tagged_bytes -= fe_ptr->stat_struct.st_size;
-	       	             statistic.disk_tagged_files--;
-		             statistic.disk_tagged_bytes -= fe_ptr->stat_struct.st_size;
-			   }
-			 }
-		       }
-		     }
-		     dir_entry->start_file = 0;
-		     dir_entry->cursor_pos = -1;
-                     DisplayFileWindow( dir_entry );
-                     RefreshWindow( file_window );
-		     DisplayDiskStatistic();
+    		     HandleTagAllDirs(dir_entry,FALSE);
 		     break;
       case 'F' & 0x1F : 
 		     RotateDirMode();
@@ -892,130 +1018,14 @@ int HandleDirWindow(DirEntry *start_dir_entry)
 		     need_dsp_help = TRUE;
 		     break;
       case 'S' & 0x1F : 
-         	     if( statistic.disk_tagged_files )
-		     {
-    		       if(dir_entry->login_flag) {
-		         dir_entry->login_flag = FALSE;
-		       } else {
-		         dir_entry->big_window  = TRUE;
-                         dir_entry->global_flag = TRUE;
-		         dir_entry->tagged_flag = TRUE;
-		         dir_entry->start_file  = 0;
-		         dir_entry->cursor_pos  = 0;
-		       }
-		       if( HandleFileWindow(dir_entry) != LOGIN_ESC )
-		       {
-		         DisplayDiskStatistic();
-		         dir_entry->start_file = 0;
-		         dir_entry->cursor_pos = -1;
-                         DisplayFileWindow( dir_entry );
-                         RefreshWindow( small_file_window );
-                         RefreshWindow( big_file_window );
-		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-		       }
-		       else
-		       {
-		         BuildDirEntryList( statistic.tree );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-			 ch = 'L';
-		       }
-		     }
-		     else
-		     {
-		       dir_entry->login_flag = FALSE;
-		       beep();
-		     }
-		     need_dsp_help = TRUE;
+		     HandleShowAllTagged(dir_entry, start_dir_entry, &need_dsp_help, &ch);
 		     break;
 
       case 'S':
-      case 's':      if( statistic.disk_matching_files )
-		     {
-    		       if(dir_entry->login_flag) {
-		         dir_entry->login_flag = FALSE;
-		       } else {
-		         dir_entry->big_window  = TRUE;
-		         dir_entry->global_flag = TRUE;
-		         dir_entry->start_file  = 0;
-		         dir_entry->cursor_pos  = 0;
-		       }
-
-		       if( HandleFileWindow(dir_entry) != LOGIN_ESC )
-		       {
-		         DisplayDiskStatistic();
-		         dir_entry->start_file = 0;
-		         dir_entry->cursor_pos = -1;
-                         DisplayFileWindow( dir_entry );
-                         RefreshWindow( small_file_window );
-                         RefreshWindow( big_file_window );
-		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-		       }
-		       else
-		       {
-		         BuildDirEntryList( statistic.tree );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-			 ch = 'L';
-		       }
-		     }
-		     else
-		     {
-		       dir_entry->login_flag = FALSE;
-		       beep();
-		     }
-		     need_dsp_help = TRUE;
+      case 's':      HandleShowAll(dir_entry, start_dir_entry, &need_dsp_help, &ch);
 		     break;
       case LF :
-      case CR :      if( dir_entry->matching_files )
-		     {
-    		       if(dir_entry->login_flag) {
-		         dir_entry->login_flag = FALSE;
-		       } else {
-		         dir_entry->global_flag = FALSE;
-                         dir_entry->tagged_flag = FALSE;
-		         dir_entry->big_window  = bypass_small_window;
-		         dir_entry->start_file  = 0;
-		         dir_entry->cursor_pos  = 0;
-		       }
-
-		       if( HandleFileWindow( dir_entry ) != LOGIN_ESC )
-                       {
-		         dir_entry->start_file = 0;
-		         dir_entry->cursor_pos = -1;
-			 DisplayFileWindow( dir_entry );
-                         RefreshWindow( small_file_window );
-                         RefreshWindow( big_file_window );
-		         BuildDirEntryList( start_dir_entry );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-			 DisplayDiskStatistic();
-		       }
-		       else
-		       {
-		         BuildDirEntryList( statistic.tree );
-                         DisplayTree( dir_window, statistic.disp_begin_pos, 
-				      statistic.disp_begin_pos + statistic.cursor_pos 
-				    );
-			 ch = 'L';
-		       }
-		       DisplayAvailBytes();
-		       need_dsp_help = TRUE;
-		     }
-		     else
-		     {
-		       dir_entry->login_flag = FALSE;
-		       beep();
-		     }
+      case CR :      HandleSwitchWindow(dir_entry, start_dir_entry, &need_dsp_help, &ch);
 		     break;
       case 'X':
       case 'x':      (void) Execute( dir_entry, NULL );
@@ -1086,6 +1096,12 @@ int HandleDirWindow(DirEntry *start_dir_entry)
                      DisplayTree( dir_window, statistic.disp_begin_pos, statistic.disp_begin_pos + statistic.cursor_pos );
 		     need_dsp_help = TRUE;
 		     break;
+
+      case 'Q' & 0x1F:
+                     need_dsp_help = TRUE;
+                     QuitTo(dir_entry);
+                     break;
+
       case 'Q':
       case 'q':      need_dsp_help = TRUE;
                      break;
@@ -1330,3 +1346,54 @@ int KeyF2Get(DirEntry *start_dir_entry,
 
 
 
+int RefreshDirWindow()
+{
+	DirEntry *de_ptr;
+	int i, n;
+	int result = -1;
+	int window_width, window_height;
+
+	de_ptr = dir_entry_list[statistic.disp_begin_pos + statistic.cursor_pos].dir_entry;
+	BuildDirEntryList( dir_entry_list[0].dir_entry );
+
+	/* Search old entry */
+	for(n=-1, i=0;i < total_dirs; i++) {
+		if(dir_entry_list[i].dir_entry == de_ptr) {
+			n = i;
+			break;
+		}
+	}
+
+	if(n == -1) {
+		/* Directory disapeared */
+      		ERROR_MSG( "Current directory disappeared" );
+		result = -1;
+	} else {
+
+		if( n != (statistic.disp_begin_pos + statistic.cursor_pos)) {
+			/* Position changed */
+			if((n - statistic.disp_begin_pos) >= 0) {
+				statistic.cursor_pos = n - statistic.disp_begin_pos;
+			} else {
+				statistic.disp_begin_pos = n;
+				statistic.cursor_pos = 0;
+			}
+		}
+	
+       		GetMaxYX(dir_window, &window_height, &window_width);
+	        while(statistic.cursor_pos >= window_height) {
+		  statistic.cursor_pos--;
+		  statistic.disp_begin_pos++;
+	        }
+		DisplayTree( dir_window, statistic.disp_begin_pos, 
+		     	statistic.disp_begin_pos + statistic.cursor_pos 
+		   	);
+	
+		DisplayAvailBytes();
+		result = 0;
+	}
+	
+	return(result);
+}
+
+
