@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/filespec.c,v 1.10 2000/05/28 17:48:40 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/filespec.c,v 1.11 2003/03/16 13:13:29 werner Exp $
  *
  * Setzt neue Datei-Spezifikation
  *
@@ -71,6 +71,13 @@ void SetMatchingParam(DirEntry *dir_entry)
 
 
 
+/***************************************************************>>
+ReadFileSpec.
+Take in the user-specified new filespec.
+As modified, it defaults to '*'; the original version offered the
+current value as default, but that's just an up-arrow away.
+Returns 0 on success, -1 on failure (empty string).
+<<***************************************************************/            
 
 int ReadFileSpec(void)
 {
@@ -80,7 +87,7 @@ int ReadFileSpec(void)
 
   ClearHelp();
 
-  (void) strcpy( buffer, statistic.file_spec );
+  (void) strcpy( buffer, "*" );
   MvAddStr( LINES - 2, 1, "New filespec:" );
   if( InputString( buffer, LINES - 2, 15, FILE_SPEC_LENGTH, "\r\033" ) == CR )
   {
