@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/filewin.c,v 1.26 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/filewin.c,v 1.29 2003/08/31 11:11:00 werner Exp $
  *
  * Funktionen zur Handhabung des FILE-Windows
  *
@@ -487,7 +487,7 @@ static void RemoveFileEntry(int entry_no)
     if( i >= entry_no ) file_entry_list[i] = file_entry_list[i+1];    
     fe_ptr = file_entry_list[i].file;
     l = strlen( fe_ptr->name );
-    max_filename_len = MAX( max_filename_len, l );
+    max_filename_len = MAX( (int)max_filename_len, l );
     if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
     {
       max_linkname_len = MAX( max_filename_len, strlen( &fe_ptr->name[l+1] ) );
@@ -516,7 +516,7 @@ static void ChangeFileEntry(void)
     if( fe_ptr )
     {
       l = strlen( fe_ptr->name );
-      max_filename_len = MAX( max_filename_len, l );
+      max_filename_len = MAX( (int)max_filename_len, l );
       if( S_ISLNK( fe_ptr->stat_struct.st_mode ) )
       {
         max_linkname_len = MAX( max_filename_len, strlen( &fe_ptr->name[l+1] ) );
@@ -979,7 +979,7 @@ static void DisplayFiles(DirEntry *de_ptr, int start_file_no, int hilight_no, in
   {
     for( y=0; y < window_height; y++ )
     {
-      if( j < file_count )
+      if( j < (int)file_count )
       {
 	if( j == hilight_no )
 	{
@@ -1003,7 +1003,7 @@ static void DisplayFiles(DirEntry *de_ptr, int start_file_no, int hilight_no, in
 
 static void fmovedown(int *start_file, int *cursor_pos, int *start_x, DirEntry *dir_entry)
 {
-   if( *start_file + *cursor_pos + 1 >= file_count )
+   if( *start_file + *cursor_pos + 1 >= (int)file_count )
    {
       /* File nicht vorhanden */
       /*----------------------*/
@@ -1102,7 +1102,7 @@ static void fmoveright(int *start_file, int *cursor_pos, int *start_x,DirEntry *
                       );
       if( hide_right < 0 ) (*start_x)--;
    }
-   else if( *start_file + *cursor_pos >= file_count - 1 )
+   else if( *start_file + *cursor_pos >= (int)file_count - 1 )
    {
       /*letzte Position erreicht */
       /*-------------------------*/
@@ -1110,7 +1110,7 @@ static void fmoveright(int *start_file, int *cursor_pos, int *start_x,DirEntry *
    }
    else 
    {
-      if( *start_file + *cursor_pos + x_step >= file_count )
+      if( *start_file + *cursor_pos + x_step >= (int)file_count )
       {
           /* voller Step nicht moeglich;
            * auf letzten Eintrag positionieren
@@ -1226,7 +1226,7 @@ static void fmoveleft(int *start_file, int *cursor_pos, int *start_x, DirEntry *
 
 static void fmovenpage(int *start_file, int *cursor_pos, int *start_x, DirEntry *dir_entry)
 {
-   if( *start_file + *cursor_pos >= file_count - 1 )
+   if( *start_file + *cursor_pos >= (int)file_count - 1 )
    {
       /*letzte Position erreicht */
       /*-------------------------*/
@@ -1246,7 +1246,7 @@ static void fmovenpage(int *start_file, int *cursor_pos, int *start_x, DirEntry
                          FALSE,
                          *start_x
                          );
-         if( *start_file + max_disp_files <= file_count - 1 )
+         if( *start_file + max_disp_files <= (int)file_count - 1 )
             *cursor_pos = max_disp_files - 1;
          else
             *cursor_pos = file_count - *start_file - 1;
@@ -1261,11 +1261,11 @@ static void fmovenpage(int *start_file, int *cursor_pos, int *start_x, DirEntry
       {
         /* Scrollen */
         /*----------*/
-        if( *start_file + *cursor_pos + max_disp_files < file_count )
+        if( *start_file + *cursor_pos + max_disp_files < (int)file_count )
            *start_file += max_disp_files;
         else
            *start_file = file_count - max_disp_files;
-        if( *start_file + max_disp_files <= file_count - 1 )
+        if( *start_file + max_disp_files <= (int)file_count - 1 )
            *cursor_pos = max_disp_files - 1;
         else
            *cursor_pos = file_count - *start_file - 1;
@@ -1546,7 +1546,7 @@ int HandleFileWindow(DirEntry *dir_entry)
       case KEY_PPAGE: fmoveppage(&dir_entry->start_file, &dir_entry->cursor_pos, &start_x, dir_entry);
 		      break;
     
-      case KEY_END  : if( dir_entry->start_file + dir_entry->cursor_pos + 1 >= file_count )
+      case KEY_END  : if( dir_entry->start_file + dir_entry->cursor_pos + 1 >= (int)file_count )
 		      {
 			/* Letzte Position erreicht */
 			/*--------------------------*/
@@ -1555,7 +1555,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 		      }
 		      else
 		      {
-			if( file_count < max_disp_files )
+			if( (int)file_count < max_disp_files )
 		        {
 			  dir_entry->start_file = 0;
 			  dir_entry->cursor_pos = file_count - 1;
@@ -1812,7 +1812,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 		      break;
       
       case 'T' & 0x1F : 
-                      for(i=0; i < file_count; i++)
+                      for(i=0; i < (int)file_count; i++)
                       {
 			fe_ptr = file_entry_list[i].file;
 			de_ptr = fe_ptr->dir_entry;
@@ -1843,7 +1843,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 
 
       case 'U' & 0x1F : 
-                      for(i=0; i < file_count; i++)
+                      for(i=0; i < (int)file_count; i++)
                       {
 			fe_ptr = file_entry_list[i].file;
 			de_ptr = fe_ptr->dir_entry;
@@ -1876,7 +1876,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 
       case ';': 
       case 't' | 0x80 : 
-                      for(i=dir_entry->start_file + dir_entry->cursor_pos; i < file_count; i++)
+                      for(i=dir_entry->start_file + dir_entry->cursor_pos; i < (int)file_count; i++)
                       {
 			fe_ptr = file_entry_list[i].file;
 			de_ptr = fe_ptr->dir_entry;
@@ -1908,7 +1908,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 
       case ':': 
       case 'u' | 0x80 : 
-                      for(i=dir_entry->start_file + dir_entry->cursor_pos; i < file_count; i++)
+                      for(i=dir_entry->start_file + dir_entry->cursor_pos; i < (int)file_count; i++)
                       {
 			fe_ptr = file_entry_list[i].file;
 			de_ptr = fe_ptr->dir_entry;
@@ -1941,7 +1941,7 @@ int HandleFileWindow(DirEntry *dir_entry)
       case 'v' :      fe_ptr = file_entry_list[dir_entry->start_file + dir_entry->cursor_pos].file;
 		      de_ptr = fe_ptr->dir_entry;
 		      (void) GetRealFileNamePath( fe_ptr, filepath );
-		      (void) View( filepath );
+		      (void) View( dir_entry, filepath );
 		      need_dsp_help = TRUE;
 		      break;
 
@@ -1950,13 +1950,14 @@ int HandleFileWindow(DirEntry *dir_entry)
 		      de_ptr = fe_ptr->dir_entry;
 		      (void) GetRealFileNamePath( fe_ptr, filepath );
 		      (void) ViewHex( filepath );
+		      need_dsp_help = TRUE;
 		      break;
       
       case 'E':
       case 'e' :      fe_ptr = file_entry_list[dir_entry->start_file + dir_entry->cursor_pos].file;
 		      de_ptr = fe_ptr->dir_entry;
 		      (void) GetFileNamePath( fe_ptr, filepath );
-		      (void) Edit( filepath );
+		      (void) Edit( de_ptr, filepath );
 		      break;
 
       case 'Y' :
@@ -2244,7 +2245,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 		        
 			if( file_count == 0 ) unput_char = ESC;
 
-			if( dir_entry->start_file + dir_entry->cursor_pos >= file_count )
+			if( dir_entry->start_file + dir_entry->cursor_pos >= (int)file_count )
 			{
 			  if( --dir_entry->cursor_pos < 0 )
 			  {
@@ -2357,7 +2358,7 @@ int HandleFileWindow(DirEntry *dir_entry)
 		          
 			if( file_count == 0 ) unput_char = ESC;
 		          
-			if( dir_entry->start_file + dir_entry->cursor_pos >= file_count )
+			if( dir_entry->start_file + dir_entry->cursor_pos >= (int)file_count )
 			{
 			  if( --dir_entry->cursor_pos < 0 )
 			  {
@@ -2658,6 +2659,14 @@ int HandleFileWindow(DirEntry *dir_entry)
 		      }
 		      break;
                       
+      case 'Q' & 0x1F:
+                      need_dsp_help = TRUE;
+                      fe_ptr = file_entry_list[dir_entry->start_file
+                                + dir_entry->cursor_pos].file;
+                      de_ptr = fe_ptr->dir_entry;
+                      QuitTo(de_ptr);
+                      break;
+
       case 'Q':
       case 'q':       need_dsp_help = TRUE;
                       Quit();
@@ -2710,7 +2719,7 @@ static void WalkTaggedFiles(int start_file,
  
   max_disp_files = window_height * max_column;
 
-  for( i=0; i < file_count && result == 0; i++ )
+  for( i=0; i < (int)file_count && result == 0; i++ )
   {
     fe_ptr = file_entry_list[i].file;
 
@@ -2787,7 +2796,7 @@ static void SilentWalkTaggedFiles( int (*fkt) (/* ??? */),
   int       result = 0;
 
   
-  for( i=0; i < file_count && result == 0; i++ )
+  for( i=0; i < (int)file_count && result == 0; i++ )
   {
     fe_ptr = file_entry_list[i].file;
 
@@ -2806,7 +2815,7 @@ static BOOL IsMatchingTaggedFiles(void)
   FileEntry *fe_ptr;
   int i;
 
-  for( i=0; i < file_count; i++)
+  for( i=0; i < (int)file_count; i++)
   {
     fe_ptr = file_entry_list[i].file;
 
@@ -2844,7 +2853,7 @@ static int DeleteTaggedFiles(int max_disp_files)
 
   if( baudrate() >= QUICK_BAUD_RATE ) typeahead( 0 );
 
-  for( i=0; i < file_count && result == 0; )
+  for( i=0; i < (int)file_count && result == 0; )
   {
     deleted = FALSE;
 
@@ -2917,7 +2926,7 @@ static void RereadWindowSize(DirEntry *dir_entry)
   max_disp_files = window_height * max_column;
 
 
-  if( dir_entry->start_file + dir_entry->cursor_pos < file_count )
+  if( dir_entry->start_file + dir_entry->cursor_pos < (int)file_count )
   {
      while( dir_entry->cursor_pos >= max_disp_files )
      {
