@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/group.c,v 1.9 1997/08/13 12:24:58 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/group.c,v 1.10 2003/06/06 19:47:26 werner Exp $
  *
  * Handhabung von Gruppen-Nummern / Namen
  *
@@ -71,7 +71,7 @@ int ReadGroupEntries(void)
     }
   }
 
-  for(i=0; i < group_count; i++)
+  for(i=0; i < (int)group_count; i++)
   {
     if( ( grp_ptr = getgrent() ) == NULL )
     {
@@ -109,9 +109,9 @@ char *GetGroupName(unsigned int gid)
 
   int i;
 
-  for( i=0; i < group_count; i++ )
+  for( i=0; i < (int)group_count; i++ )
   {
-    if( group_array[i].gid == gid )
+    if( group_array[i].gid == (int)gid )
       return( group_array[i].name );
   }
 
@@ -139,7 +139,7 @@ int GetGroupId(char *name)
 
   int i;
 
-  for( i=0; i < group_count; i++ )
+  for( i=0; i < (int)group_count; i++ )
   {
     if( !strcmp( name, group_array[i].name ) )
       return( (int) group_array[i].gid );
