@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/mkdir.c,v 1.14 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/mkdir.c,v 1.15 2003/06/06 19:47:26 werner Exp $
  *
  * Erstellen von Verzeichnissen
  *
@@ -181,7 +181,7 @@ int MakePath( DirEntry *tree, char *dir_path, DirEntry **dest_dir_entry )
   int      n;
   int      result = -1;
 
-  (void) strcpy( path, dir_path );
+  NormPath( dir_path, path );
   *dest_dir_entry = NULL;
 
   n = strlen( tree->name );
