@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/move.c,v 1.12 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/move.c,v 1.13 2003/08/31 11:11:00 werner Exp $
  *
  * Beschreibung : Bewegen von Dateien
  *
@@ -183,7 +183,9 @@ int MoveFile(FileEntry *fe_ptr,
 
 FNC_XIT:
  
+  move( LINES - 3, 1 ); clrtoeol();
   move( LINES - 2, 1 ); clrtoeol();
+  move( LINES - 1, 1 ); clrtoeol();
 
   return( result );
 }
@@ -195,7 +197,6 @@ FNC_XIT:
 int GetMoveParameter(char *from_file, char *to_file, char *to_dir)
 {
   char buffer[PATH_LENGTH+1];
-  int  x;
 
   if( from_file == NULL )
   {
@@ -207,30 +208,19 @@ int GetMoveParameter(char *from_file, char *to_file, char *to_dir)
     (void) strcpy( to_file, from_file );
   }
 
-  (void) sprintf( buffer, "MOVE %s AS", from_file );
+  (void) sprintf( buffer, "MOVE %s", from_file );
 
   ClearHelp();
 
-  MvAddStr( LINES - 2, 1, buffer );
-
-  if( InputString( to_file, 
-		   LINES - 2, 
-		   strlen(buffer) + 2, 
-		   COLS - strlen(buffer) - 3, 
-		   "\r\033" ) != CR
-    )
-  {
-    return( -1 );
+  MvAddStr( LINES - 3, 1, buffer );
+  MvAddStr( LINES - 2, 1, "AS  " );
+  if( InputString( to_file, LINES - 2, 6, COLS - 6, "\r\033" ) == CR ) {
+    MvAddStr( LINES - 1, 1, "TO   " );
+    if( InputString( to_dir, LINES - 1, 6, COLS - 6, "\r\033" ) == CR )
+    return( 0 );
   }
-
-  x = 1 + strlen( buffer ) + 1 + strlen( to_file ) + 1;
-
-  MvAddStr( LINES - 2, x, "TO " ); 
-  x += 3;
-  if( InputString( to_dir, LINES - 2, x, COLS - x - 1, "\r\033" ) != CR ) 
-    return( -1 );
-
-  return( 0 );
+  ClearHelp();
+  return( -1 );
 }
 
 
