@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/passwd.c,v 1.9 1997/08/13 12:24:58 werner Rel $
+ * $Header: /usr/local/cvsroot/utils/ytree/passwd.c,v 1.10 2003/06/06 19:47:26 werner Exp $
  *
  * Handhabung von User-Nummern / Namen
  *
@@ -70,7 +70,7 @@ int ReadPasswdEntries(void)
     }
   }
 
-  for(i=0; i < passwd_count; i++)
+  for(i=0; i < (int)passwd_count; i++)
   {
     if( ( pwd_ptr = getpwent() ) == NULL )
     {
@@ -109,9 +109,9 @@ char *GetPasswdName(unsigned int uid)
 
   int i;
 
-  for( i=0; i < passwd_count; i++ )
+  for( i=0; i < (int)passwd_count; i++ )
   {
-    if( passwd_array[i].uid == uid )
+    if( passwd_array[i].uid == (int)uid )
       return( passwd_array[i].name );
   }
 
@@ -139,7 +139,7 @@ int GetPasswdUid(char *name)
 
   int i;
 
-  for( i=0; i < passwd_count; i++ )
+  for( i=0; i < (int)passwd_count; i++ )
   {
     if( !strcmp( name, passwd_array[i].name ) )
       return( (int) passwd_array[i].uid );
