@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/pipe.c,v 1.13 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/pipe.c,v 1.14 2003/03/16 13:13:29 werner Exp $
  *
  * Umlenken von Datei-Inhalten zu einem Kommando
  *
@@ -44,9 +44,9 @@ int Pipe(DirEntry *dir_entry, FileEntry *file_entry)
   {
     move( LINES - 2, 1 ); clrtoeol();
     
-    if( getcwd( cwd, PATH_LENGTH ) == NULL )
+    if( Getcwd( cwd, PATH_LENGTH ) == NULL )
     {
-      WARNING( "getcwd failed*\".\"assumed" );
+      WARNING( "Getcwd failed*\".\"assumed" );
       (void) strcpy( cwd, "." );
     }
 
