@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/util.c,v 1.26 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/util.c,v 1.29 2003/08/31 11:11:00 werner Exp $
  *
  * Diverse Hilfsfunktionen
  *
@@ -20,6 +20,7 @@ typedef struct
 
 
 static Extension2Method file_extensions[] = FILE_EXTENSIONS;
+static char *GNU_getcwd(void);
 
 
 char *GetPath(DirEntry *dir_entry, char *buffer)
@@ -104,9 +105,9 @@ int GetDirEntry(DirEntry *tree,
   *to_path   = '\0';
 
    strcpy(to_path, dir_path);
-  if( getcwd( current_path, sizeof( current_path ) - 2 ) == NULL )
+  if( Getcwd( current_path, sizeof( current_path ) - 2 ) == NULL )
   {
-    (void) sprintf( message, "getcwd failed*%s", strerror(errno) );
+    (void) sprintf( message, "Getcwd failed*%s", strerror(errno) );
     ERROR_MSG( message );
     return( -1 );
   }
@@ -130,7 +131,7 @@ int GetDirEntry(DirEntry *tree,
   }
 
   if( *dir_path != FILE_SEPARATOR_CHAR ) {
-    (void) getcwd( dest_path, sizeof( dest_path ) - 2 );
+    (void) Getcwd( dest_path, sizeof( dest_path ) - 2 );
     (void) strcpy( to_path, dest_path );
   } else {
     strcpy(dest_path, dir_path);
@@ -261,277 +262,6 @@ char *GetAttributes(unsigned short modus, char *buffer)
 
 
 
-
-
-/***************************************************************************
- * InputStr                                                                *
- * Liest eine Zeichenkette an Position (y,x) mit der max. Laenge length    *
- * Vorschlagswert fuer die Eingabe ist s selbst                            *
- * Zurueckgegeben wird das Zeichen, mit dem die Eingabe beendet wurde      *
- ***************************************************************************/
-
-
-int InputString(char *s, int y, int x, int length, char *term)
-                               /* Ein- und Ausgabestring              */
-                               /* Position auf Bildschirm             */
-                               /* max. Laenge                         */
-                               /* Menge von Terminierungszeichen      */
-{
-  int p;                       /* Aktuelle Position                   */
-  int c1;                      /* Gelesenes Zeichen                   */
-  int i;                       /* Laufvariable                        */
-  char *pp;
-  BOOL key_pressed = FALSE;
-  char path[PATH_LENGTH + 1];
-
-  static BOOL insert_flag = TRUE;
-
-  /* Feld gefuellt ausgeben */
-  /*------------------------*/
-  print_time = FALSE;
-  curs_set(1);
-  MvAddStr( y, x, s );
-  leaveok(stdscr, FALSE);
-  
-  for(i=strlen(s); i < length; i++)
-    addch( '_' );
-
-
-  p = 0;
-
-  do  /* until termkey pressed */
-  {
-    wmove( stdscr, y, x+p );
-
-    RefreshWindow( stdscr );
-    doupdate();
-
-    c1 = Getch();
-
-    if( c1 >= ' ' && c1 < 0xff && c1 != 127) 
-    {
-      /* Darstellbares Zeichen */
-      /*-----------------------*/
-
-      if( key_pressed == FALSE )
-      {
-	s[0] = '\0';
-        
-	move( y, x );
-        for(i=0; i < length; i++)
-          addch( '_' );
-	move( y, x );
-      }
-
-      if( p < length )
-      {
-        /* Platz zum einfuegen vorhanden */
-        /*-------------------------------*/
-
-        if( insert_flag )
-        {
-          /* Restliche Zeichen eine Position vorruecken */
-          /*--------------------------------------------*/
-
-          if( strlen( s ) == length ) s[length-1] = '\0';
-          for(i=strlen(s); i >= p; i-- )
-            s[i+1] = s[i];
-        }
-        else
-        {
-          /* Simples ueberschreiben */
-          /*------------------------*/
-
-          if( s[p] == '\0' )
-            s[p+1] = '\0';
-        }
-
-        s[p] = (char) c1;
-
-        addstr( &s[p] );
-        p++;
-      }
-      else
-      {
-        /* Zeile voll */
-        /*------------*/
-
-        beep();
-      }
-    }
-    else
-    {
-      /* Steuerzeichen */
-      /*---------------*/
-
-      switch( c1 )
-      {
-        case -1             : break;
-
-        case KEY_LEFT       : if( p > 0 )
-                                p--;
-                              else
-                                beep();
-                              break;
-        case KEY_RIGHT      : if( p < (int) strlen( s ) )
-                                p++;
-                              else
-                                beep();
-                              break;
-	case KEY_UP         : 
-                             if(( pp = GetHistory()) == NULL) {
-			       break;
-			     }
-                             if(*pp)
-                             {
-                               strncpy(s, pp, length);
-			       s[length] = '\0';
-			       p = strlen(s);
-                               MvAddStr( y, x, s );
-                               for(i=p; i < length; i++)
-                                 addch( '_' );
-                               RefreshWindow( stdscr );
-                               doupdate();
-                             }
-                             break;
-
-        case KEY_HOME       : p = 0;
-                              break;
-        case KEY_END        : p = strlen( s );
-                              break;
-        case KEY_DC         : if( p < (int) strlen( s ) )
-                              {
-                                (void) strcpy( s+p, s+p+1 );
-                                printw( "%s_", s+p );
-                              }
-                              break;
-	case 0x08           :
-        case 0x7F           :
-        case KEY_BACKSPACE  : if( p > 0 )
-                              {
-                                (void) strcpy( s+p-1, s+p );
-                                printw( "\b%s_", s+p-1 );
-                                p--;
-                              }
-                              else
-                                beep();
-                              break;
-        case KEY_DL         : for(i=0; i < (int) strlen(s) - p; i++)
-			        addch( '_' );
-                              s[p] = '\0';
-                              break;
-	case KEY_EIC        :
-        case KEY_IC         : insert_flag ^= TRUE; 
-                              break;
-
-#ifdef KEY_F
-        case KEY_F(2)       : 
-#endif
-        case 'F' & 0x1f     : if(KeyF2Get( statistic.tree,
-                                           statistic.disp_begin_pos,
-                                           statistic.cursor_pos, path)) 
-                              {
-			        /* beep(); */
-				break;
-			      }
-                              if(*path)
-                              {
-                                strncpy(s, path, length);
-			        s[length] = '\0';
-			        p = strlen(s);
-                                MvAddStr( y, x, s );
-                                for(i=p; i < length; i++)
-                                  addch( '_' );
-                                RefreshWindow( stdscr );
-                                doupdate();
-                              }
-                              break;
-
-        default             : if( c1 == LF ) c1 = CR;
-                              if( !strrchr( term, c1 ) )
-                                beep();
-                              break;
-      }
-    }
-    key_pressed = TRUE;
-  } while( c1 != -1 && !strrchr( term, c1 ) );
-
-  p = strlen( s );
-
-  move( y, x + p );
-
-  for(i=0; i < length - p; i++ )
-   addch( ' ' );
-
-  move( y, x );
-  leaveok( stdscr, TRUE);
-  curs_set(0);
-  print_time = TRUE;
-  InsHistory( s );
-  return( c1 );
-}
-
-
-
-
-
-int InputChoise(char *msg, char *term)
-{
-  int  c;
-
-  ClearHelp();
-
-  curs_set(1);
-  leaveok(stdscr, FALSE);
-  mvprintw( LINES - 2, 1, msg );
-  RefreshWindow( stdscr );
-  doupdate();
-  do
-  {
-    c = Getch();
-    if(c >= 0)
-      if( islower( c ) ) c = toupper( c );
-  } while( c != -1 && !strchr( term, c ) );
-  
-  if(c >= 0)
-    echochar( c );
-
-  move( LINES - 2, 1 ); clrtoeol();
-  leaveok(stdscr, TRUE);
-  curs_set(0);
-
-  return( c );
-}
-
-
-
-
-
-int GetTapeDeviceName( void )
-{
-  int  result;
-  char path[PATH_LENGTH+1];
-
-  result = -1;
-
-  ClearHelp();
-
-  (void) strcpy( path, statistic.tape_name );
-
-  MvAddStr( LINES - 2, 1, "Tape-Device:" );
-  if( InputString( path, LINES - 2, 14, COLS - 15, "\r\033" ) == CR )
-  {
-    result = 0;
-    (void) strcpy( statistic.tape_name, path );
-  }
-
-  move( LINES - 2, 1 ); clrtoeol();
-
-  return( result );
-}
-
-
-
 char *CTime(time_t f_time, char *buffer)
 {
   char   *cptr;
@@ -571,8 +301,11 @@ void PrintSpecialString(WINDOW *win, int y, int x, char *str, int color)
     return;
   }
 
+  wmove( win, y, x);
+
   for( ; *str; str++ )
   {
+    if ( (!iscntrl(*str)) || (!isspace(*str)) || (*str=' ') ) 
     switch( *str )
     {
       case '1': ch = ACS_ULCORNER; break;
@@ -588,13 +321,39 @@ void PrintSpecialString(WINDOW *win, int y, int x, char *str, int color)
       case '-': ch = ACS_HLINE;    break;
       default:  ch = PRINT(*str);
     }
+    else
+            ch = ACS_BLOCK;
 #ifdef COLOR_SUPPORT
     ch |= COLOR_PAIR(color) | A_BOLD;
 #endif /* COLOR_SUPPORT */
-    mvwaddch( win, y, x++, ch );
+    waddch( win, ch );
   }
 }
 
+
+void Print(WINDOW *win, int y, int x, char *str, int color)
+{
+ int ch;
+
+  if(x < 0 || y < 0) {
+     /* screen too small */
+    return;
+  }
+  wmove(win, y, x);
+  for( ; *str; str++ )
+  {
+    ch = PRINT((int) *str);
+   
+#ifdef COLOR_SUPPORT
+    ch |= (COLOR_PAIR(color) | A_BOLD);
+#else
+    ch |= color;
+#endif
+    waddch(win, ch );
+  }
+}
+
+
 void PrintOptions(WINDOW *win, int y, int x, char *str)
 {
   int ch;
@@ -750,6 +509,25 @@ char *CutFilename(char *dest, char *src, unsigned int max_len)
   }
 }
 
+/*****************************************************************************
+ *                              CutPathname                                  *
+ *****************************************************************************/
+
+char *CutPathname(char *dest, char *src, unsigned int max_len)
+{
+  unsigned int l;
+
+  l = strlen(src);
+
+  if( l <= max_len )
+    return( strcpy( dest, src ) );
+  else
+  {
+    (void) strcpy( dest, "..." );
+    (void) strncat( dest, &src[l - max_len + 3], max_len - 3 );
+    return( dest );
+  }
+}
 
 
 /*****************************************************************************
@@ -790,23 +568,6 @@ void Fnsplit(char *path, char *dir, char *name)
 
 
 
-void HitReturnToContinue(void)
-{
-#ifndef XCURSES
-  curs_set(1);
-  vidattr( A_REVERSE );
-  putp( "[Hit return to continue]" );
-  vidattr( 0 );
-  (void) fflush( stdout );
-  (void) Getch();
-#endif /* XCURSES */
-  curs_set(0);
-  doupdate();
-}
-
-
-
-
 int BuildFilename( char *in_filename,
 		   char *pattern,
 		   char *out_filename
@@ -843,7 +604,7 @@ int GetFileMethod( char *filename )
   l = strlen( filename );
 
   for( i=0; 
-       i < sizeof( file_extensions ) / sizeof( file_extensions[0] ); 
+       i < (int)(sizeof( file_extensions ) / sizeof( file_extensions[0] )); 
        i++ 
      )
   {
@@ -857,38 +618,6 @@ int GetFileMethod( char *filename )
 
 
 
-
-BOOL KeyPressed()
-{
-  BOOL pressed = FALSE;
-
-#if !defined( linux ) || !defined( TERMCAP )
-  nodelay( stdscr, TRUE );
-  if( wgetch( stdscr ) != ERR ) pressed = TRUE;
-  nodelay( stdscr, FALSE );
-#endif /* linux/TERMCAP */
-
-  return( pressed );
-}
-
-
-BOOL EscapeKeyPressed()
-{
-  BOOL pressed = FALSE;
-  int  c;
-
-#if !defined( linux ) || !defined( TERMCAP )
-  nodelay( stdscr, TRUE );
-  if( ( c = wgetch( stdscr ) ) != ERR ) pressed = TRUE;
-  nodelay( stdscr, FALSE );
-#endif /* linux/TERMCAP */
-
-  return( ( pressed && c == ESC ) ? TRUE : FALSE );
-}
-
-
-
-
 #ifdef __NeXT__
 
 char *getcwd(char *dest, int len)
@@ -927,41 +656,6 @@ char *getcwd(char *dest, int len)
 #endif
 
 
-#ifdef VI_KEYS
-
-int ViKey( int ch )
-{
-  switch( ch )
-  {
-    case VI_KEY_UP:    ch = KEY_UP;    break;
-    case VI_KEY_DOWN:  ch = KEY_DOWN;  break;
-    case VI_KEY_RIGHT: ch = KEY_RIGHT; break;
-    case VI_KEY_LEFT:  ch = KEY_LEFT;  break;
-    case VI_KEY_PPAGE: ch = KEY_PPAGE; break;
-    case VI_KEY_NPAGE: ch = KEY_NPAGE; break;
-  }
-  return(ch);
-}
-
-#endif /* VI_KEYS */
-
-  
-#ifdef _IBMR2
-#undef wgetch
-
-int AixWgetch( WINDOW *w )
-{
-  int c;
-  
-  if( ( c = wgetch( w ) ) == KEY_ENTER ) c = LF;
-
-  return( c );
-}
-
-#endif
-
-
-
 void NormPath( char *in_path, char *out_path )
 {
   char *s, *d;
@@ -1342,22 +1036,6 @@ LONGLONG AtoLL(char *cptr)
 }
 
 
-int Getch()
-{
-  int c;
-
-  c = getch();
-
-#ifdef KEY_RESIZE
-  if(c == KEY_RESIZE) {
-    resize_request = TRUE;
-    c = -1;
-  }
-#endif
-
-  return(c);
-}
-
 
 
 #ifndef HAVE_STRERROR
@@ -1376,3 +1054,34 @@ const char *StrError(int errnum)
 #endif /* HAVE_STRERROR */
 
 
+/*****************************************************************************
+ *                              Getcwd                                       *
+ *****************************************************************************/
+
+
+char *Getcwd(char *buffer, unsigned int size)
+{
+  if(size == 0)
+    return(GNU_getcwd());
+
+  return(getcwd(buffer, size));
+}
+
+
+static char *GNU_getcwd()
+{
+  unsigned int size = 100;
+
+  while (1)
+    {
+      char *buffer = (char *) malloc (size);
+      if (getcwd (buffer, size) == buffer)
+        return buffer;
+      free (buffer);
+      if (errno != ERANGE)
+        return 0;
+      size *= 2;
+    }
+}
+
+
