@@ -1,6 +1,6 @@
 /***************************************************************************
  *
- * $Header: /usr/local/cvsroot/utils/ytree/ytree.h,v 1.29 2001/06/15 16:36:36 werner Exp $
+ * $Header: /usr/local/cvsroot/utils/ytree/ytree.h,v 1.34 2003/08/31 11:11:00 werner Exp $
  *
  * Header-Datei fuer YTREE
  *
@@ -78,6 +78,11 @@ extern char *getcwd();
 #include <termcap.h>
 #endif
 
+#ifdef openbsd
+#define STATFS(a, b, c, d )     statfs( a, b )
+#define LONGLONG		long long
+#define HAS_LONGLONG		1
+#else
 #ifdef __FreeBSD__
 #define STATFS(a, b, c, d )     statfs( a, b )
 #define LONGLONG		long long
@@ -140,6 +145,7 @@ extern char *getcwd();
 #endif /* SVR3 */
 #endif /* OSF1 */
 #endif /* __FreeBSD__ */
+#endif /* openbsd */
 
 
 /* Some handy macros... */
@@ -409,6 +415,9 @@ extern char *getcwd();
 #define WINHST_COLOR    14
 #define HST_COLOR       15
 #define HIHST_COLOR     16
+#define WINMTCH_COLOR   14
+#define MTCH_COLOR      15
+#define HIMTCH_COLOR    16
 #define GLOBAL_COLOR    17
 #define HIGLOBAL_COLOR  18
 
@@ -607,7 +616,7 @@ extern char *getcwd();
 #define DIR_WINDOW_X         1  
 #define DIR_WINDOW_Y         2 
 #define DIR_WINDOW_WIDTH     (COLS - 23)
-#define DIR_WINDOW_HEIGHT    (LINES * 8 / 14 )
+#define DIR_WINDOW_HEIGHT    ((LINES * 8 / 14)-1)
 
 #define F2_WINDOW_X          DIR_WINDOW_X
 #define F2_WINDOW_Y          DIR_WINDOW_Y
@@ -617,12 +626,12 @@ extern char *getcwd();
 #define FILE_WINDOW_1_X      1
 #define FILE_WINDOW_1_Y      DIR_WINDOW_HEIGHT + 3
 #define FILE_WINDOW_1_WIDTH  (COLS - 23)
-#define FILE_WINDOW_1_HEIGHT (LINES - 6 - DIR_WINDOW_HEIGHT )
+#define FILE_WINDOW_1_HEIGHT (LINES - DIR_WINDOW_HEIGHT - 7 )
 
 #define FILE_WINDOW_2_X      1
 #define FILE_WINDOW_2_Y      2
 #define FILE_WINDOW_2_WIDTH  (COLS - 23)
-#define FILE_WINDOW_2_HEIGHT (LINES - 5) 
+#define FILE_WINDOW_2_HEIGHT (LINES - 6) 
 
 #define ERROR_WINDOW_WIDTH   40
 #define ERROR_WINDOW_HEIGHT  10
@@ -632,7 +641,12 @@ extern char *getcwd();
 #define HISTORY_WINDOW_X       1
 #define HISTORY_WINDOW_Y       2
 #define HISTORY_WINDOW_WIDTH   (COLS - 23)
-#define HISTORY_WINDOW_HEIGHT  (LINES - 5)
+#define HISTORY_WINDOW_HEIGHT  (LINES - 6)
+
+#define MATCHES_WINDOW_X       1
+#define MATCHES_WINDOW_Y       2
+#define MATCHES_WINDOW_WIDTH   (COLS - 23)
+#define MATCHES_WINDOW_HEIGHT  (LINES - 6)
 
 #define TIME_WINDOW_X        ((COLS > 18) ? (COLS - 18) : 1)
 #define TIME_WINDOW_Y        1
@@ -842,6 +856,7 @@ extern WINDOW *big_file_window;
 extern WINDOW *file_window;
 extern WINDOW *error_window;
 extern WINDOW *history_window;
+extern WINDOW *matches_window;
 extern WINDOW *f2_window;
 extern WINDOW *time_window;
 
@@ -855,6 +870,7 @@ extern BOOL      resize_request;
 extern char      number_seperator;
 extern BOOL      bypass_small_window;
 extern char      *initial_directory;
+extern char 	 builtin_hexdump_cmd[];
 
 
 #if defined(ultrix)
@@ -916,11 +932,13 @@ extern char *GetRealFileNamePath(FileEntry *file_entry, char *buffer);
 extern int  SystemCall(char *command_line);
 extern int  QuerySystemCall(char *command_line);
 extern int  SilentSystemCall(char *command_line);
-extern int  View(char *file_path);
+extern int  View(DirEntry * dir_entry, char *file_path);
 extern int  ViewHex(char *file_path);
-extern int  Edit(char *file_path);
+extern int  InternalView(char *file_path);
+extern int  Edit(DirEntry * dir_entry, char *file_path);
 extern void DisplayAvailBytes(void);
 extern void DisplayFileSpec(void);
+extern void QuitTo(DirEntry * dir_entry);
 extern void Quit(void);
 extern int  ReadFileSpec(void);
 extern int  InputString(char *s, int y, int x, int length, char *term);
@@ -971,10 +989,12 @@ extern char *CTime(time_t f_time, char *buffer);
 extern int  LoginDisk(char *path);
 extern int  GetNewLoginPath(char *path);
 extern void PrintSpecialString(WINDOW *win, int y, int x, char *str, int color);
+extern void Print(WINDOW *, int, int, char *, int);
 extern void PrintOptions(WINDOW *,int, int, char *);
 extern void PrintMenuOptions(WINDOW *,int, int, char *, int, int);
 extern char *FormFilename(char *dest, char *src, unsigned int max_len);
 extern char *CutFilename(char *dest, char *src, unsigned int max_len);
+extern char *CutPathname(char *dest, char *src, unsigned int max_len);
 extern void   Fnsplit(char *path, char *dir, char *name);
 extern void MakeExtractCommandLine(char *command_line, char *path, char *file, char *cmd);
 extern int MakeDirectory(DirEntry *father_dir_entry);
@@ -1007,6 +1027,7 @@ extern char *GetHistory(void);
 extern void InsHistory(char *new_hist);
 extern void ReadHistory(char *filename);
 extern void SaveHistory(char *filename);
+extern char *GetMatches(char *);
 extern int  KeyF2Get(DirEntry *start_dir_entry,
                int disp_begin_pos,
                int cursor_pos,
@@ -1025,6 +1046,7 @@ extern int Strrcmp(char *s1, char* s2);
 extern char *Strdup(char *s);
 extern char *GetExtViewer(char *filename);
 extern void InitClock(void);
+extern void SuspendClock(void);
 extern char *GetExtension(char *filename);
 extern void StrCp(char *dest, const char *src);
 extern int  BuildUserFileEntry(FileEntry *fe_ptr, 
@@ -1041,5 +1063,6 @@ extern int  FileUserMode(FileEntryList *file_entry_list, int ch);
 extern char *GetUserFileAction(int chkey, int *pchremap);
 extern char *GetUserDirAction(int chkey, int *pchremap);
 extern BOOL IsUserActionDefined(void);
-
+extern char *Getcwd(char *buffer, unsigned int len);
+extern int  RefreshDirWindow();
 
